@@ -3,7 +3,7 @@
  *
  * Tcl command execution trace command.
  *-----------------------------------------------------------------------------
- * Copyright 1991-1999 Karl Lehenbauer and Mark Diekhans.
+ * Copyright 1991-1996 Karl Lehenbauer and Mark Diekhans.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation for any purpose and without fee is hereby granted, provided
@@ -12,7 +12,7 @@
  * software for any purpose.  It is provided "as is" without express or
  * implied warranty.
  *-----------------------------------------------------------------------------
- * $Id: tclXdebug.c,v 8.10 1999/03/31 06:37:43 markd Exp $
+ * $Id: tclXdebug.c,v 7.0 1996/06/16 05:30:11 markd Exp $
  *-----------------------------------------------------------------------------
  */
 
@@ -25,17 +25,15 @@
 #define CMD_TRUNCATE_SIZE 60
 
 typedef struct traceInfo_t {
-    Tcl_Interp       *interp;
-    Tcl_Trace         traceId;
-    int               inTrace;
-    int               noEval;
-    int               noTruncate;
-    int               procCalls;
-    int               depth;
-    char             *callback;
-    Tcl_Obj          *errorStatePtr;
-    Tcl_AsyncHandler  errorAsyncHandler;
-    Tcl_Channel       channel;
+    Tcl_Interp *interp;
+    Tcl_Trace   traceHolder;
+    int         inTrace;
+    int         noEval;
+    int         noTruncate;
+    int         procCalls;
+    int         depth;
+    char       *callback;
+    Tcl_Channel channel;
     } traceInfo_t, *traceInfo_pt;
 
 /*
@@ -63,11 +61,6 @@ TraceCode  _ANSI_ARGS_((traceInfo_pt infoPtr,
                         int          argc,
                         char       **argv));
 
-static int
-TraceCallbackErrorHandler _ANSI_ARGS_((ClientData  clientData,
-                                       Tcl_Interp *interp,
-                                       int         code));
-
 static void
 TraceCallBack _ANSI_ARGS_((Tcl_Interp   *interp,
                            traceInfo_pt  infoPtr,
@@ -86,18 +79,13 @@ CmdTraceRoutine _ANSI_ARGS_((ClientData    clientData,
                              int           argc,
                              char        **argv));
 
-static int
-TclX_CmdtraceObjCmd _ANSI_ARGS_((ClientData clientData, 
-                                 Tcl_Interp *interp,
-                                 int objc,
-                                 Tcl_Obj *CONST objv[]));
-
 static void
 DebugCleanUp _ANSI_ARGS_((ClientData  clientData,
                           Tcl_Interp *interp));
 
 
-/*-----------------------------------------------------------------------------
+/*
+ *-----------------------------------------------------------------------------
  * TraceDelete --
  *
  *   Delete the trace if active, reseting the structure.
@@ -108,22 +96,19 @@ TraceDelete (interp, infoPtr)
     Tcl_Interp   *interp;
     traceInfo_pt  infoPtr;
 {
-    if (infoPtr->traceId != NULL) {
-        Tcl_DeleteTrace (interp, infoPtr->traceId);
+    if (infoPtr->traceHolder != NULL) {
+        Tcl_DeleteTrace (interp, infoPtr->traceHolder);
         infoPtr->depth = 0;
-        infoPtr->traceId = NULL;
+        infoPtr->traceHolder = NULL;
         if (infoPtr->callback != NULL) {
             ckfree (infoPtr->callback);
             infoPtr->callback = NULL;
         }
     }
-    if (infoPtr->errorAsyncHandler != NULL) {
-        Tcl_AsyncDelete (infoPtr->errorAsyncHandler);
-        infoPtr->errorAsyncHandler = NULL;
-    }
 }
 
-/*-----------------------------------------------------------------------------
+/*
+ *-----------------------------------------------------------------------------
  * PrintStr --
  *
  *     Print an string, truncating it to the specified number of characters.
@@ -148,18 +133,19 @@ PrintStr (channel, string, numChars, quoted)
             Tcl_Write (channel, &(string [idx]), 1);
         }
     }
-    if (numChars < (int) strlen (string))
+    if (numChars < strlen (string))
         Tcl_Write (channel, "...", 3);
     if (quoted) 
         Tcl_Write (channel, "}", 1);
 }
 
-/*-----------------------------------------------------------------------------
+/*
+ *-----------------------------------------------------------------------------
  * PrintArg --
  *
  *   Print an argument string, truncating and adding "..." if its longer
  * then ARG_TRUNCATE_SIZE.  If the string contains white spaces, quote
- * it with braces.
+ * it with angle brackets.
  *-----------------------------------------------------------------------------
  */
 static void
@@ -187,7 +173,8 @@ PrintArg (channel, argStr, noTruncate)
     PrintStr (channel, argStr, printLen, quoted);
 }
 
-/*-----------------------------------------------------------------------------
+/*
+ *-----------------------------------------------------------------------------
  * TraceCode --
  *
  *   Print out a trace of a code line.  Level is used for indenting
@@ -232,39 +219,9 @@ TraceCode (infoPtr, level, command, argc, argv)
     TclX_WriteNL (infoPtr->channel);
     Tcl_Flush (infoPtr->channel);
 }
-
 
-/*-----------------------------------------------------------------------------
- * TraceCallbackErrorHandler --
- *
- *   Async handler that processes an callback error.  Generates either an
- * immediate or background error.
+/*
  *-----------------------------------------------------------------------------
- */
-static int
-TraceCallbackErrorHandler (clientData, interp, code)
-    ClientData  clientData;
-    Tcl_Interp *interp;
-    int         code;
-{
-    traceInfo_pt infoPtr = (traceInfo_pt) clientData;
-
-    /*
-     * Put back error message and state.  If not interp passed in, the error
-     * is handled in the background.
-     */
-    TclX_RestoreResultErrorInfo (infoPtr->interp, infoPtr->errorStatePtr);
-    infoPtr->errorStatePtr = NULL;
-    if (interp == NULL) {
-        Tcl_BackgroundError (infoPtr->interp);
-    }    
-    
-    TraceDelete (interp, infoPtr);
-
-    return TCL_ERROR;
-}
-
-/*-----------------------------------------------------------------------------
  * TraceCallBack --
  *
  *   Build and call a callback for the command that was just executed. The
@@ -278,9 +235,8 @@ TraceCallbackErrorHandler (clientData, interp, code)
  * The code should allow for additional substitution of arguments in future
  * versions (such as a procedure with args as the last argument).  The value
  * of result, errorInfo and errorCode are preserved.  All other state must be
- * preserved by the procedure.  An error will result in an error being flagged
- * in the control block and async mark being called to handle the error
- * once the command has completed.
+ * preserved by the procedure.  A error will result in information being dumped
+ * to stderr and the trace deleted.
  *-----------------------------------------------------------------------------
  */
 static void
@@ -293,12 +249,14 @@ TraceCallBack (interp, infoPtr, level, command, argc, argv)
     char        **argv;
 {
     Interp       *iPtr = (Interp *) interp;
-    Tcl_DString   callback;
-    Tcl_Obj      *saveObjPtr;
-    char         *cmdList;
+    Tcl_DString   callback, resultBuf, errorInfoBuf, errorCodeBuf;
+    char         *cmdList, *errorInfo, *errorCode;
     char          numBuf [32];
 
     Tcl_DStringInit (&callback);
+    Tcl_DStringInit (&resultBuf);
+    Tcl_DStringInit (&errorInfoBuf);
+    Tcl_DStringInit (&errorCodeBuf);
 
     /*
      * Build the command to evaluate.
@@ -322,25 +280,61 @@ TraceCallBack (interp, infoPtr, level, command, argc, argv)
              iPtr->varFramePtr->level));
     Tcl_DStringAppendElement (&callback, numBuf);
 
-    saveObjPtr = TclX_SaveResultErrorInfo (interp);
+    /*
+     * Save errorInfo and errorCode.
+     */
+    Tcl_DStringGetResult (interp, &resultBuf);
+    errorInfo = Tcl_GetVar (interp, "errorInfo", TCL_GLOBAL_ONLY);
+    if (errorInfo != NULL)
+        Tcl_DStringAppend (&errorInfoBuf, errorInfo, -1);
+    errorCode = Tcl_GetVar (interp, "errorCode", TCL_GLOBAL_ONLY);
+    if (errorCode != NULL)
+        Tcl_DStringAppend (&errorCodeBuf, errorCode, -1);
 
     /*
-     * Evaluate the command.  If an error occurs, set up the handler to be
-     * called when its possible.
+     * Evaluate the command.  If an error occurs, dump something to stderr
+     * and delete the trace.  There is no way to return an error at this
+     * point.
      */
     if (Tcl_Eval (interp, Tcl_DStringValue (&callback)) == TCL_ERROR) {
-        Tcl_AddObjErrorInfo (interp, "\n    (\"cmdtrace\" callback command)",
-                             -1);
-        infoPtr->errorStatePtr = TclX_SaveResultErrorInfo (interp);
-        Tcl_AsyncMark (infoPtr->errorAsyncHandler);
+        Tcl_Channel stderrChan;
+
+        Tcl_AddErrorInfo (interp, "\n    (\"cmdtrace\" callback command)");
+        
+        stderrChan = Tcl_GetStdChannel (TCL_STDERR);
+        if (stderrChan != NULL) {
+            TclX_WriteStr (stderrChan, 
+                           "cmdtrace callback command error: errorCode = ");
+            TclX_WriteStr (stderrChan,
+                           Tcl_GetVar (interp, "errorCode", TCL_GLOBAL_ONLY));
+            TclX_WriteNL (stderrChan);
+            TclX_WriteStr (stderrChan,
+                           Tcl_GetVar (interp, "errorInfo", TCL_GLOBAL_ONLY));
+            TclX_WriteNL (stderrChan);
+            Tcl_Flush (stderrChan);
+        }
+        TraceDelete (interp, infoPtr);
     }
 
-    TclX_RestoreResultErrorInfo (interp, saveObjPtr);
+    /*
+     * Restore errorInfo and errorCode.
+     */
+    if (errorInfo != NULL)
+        Tcl_SetVar (interp, "errorInfo", Tcl_DStringValue (&errorInfoBuf),
+                    TCL_GLOBAL_ONLY);
+    if (errorCode != NULL)
+        Tcl_SetVar (interp, "errorCode", Tcl_DStringValue (&errorCodeBuf),
+                    TCL_GLOBAL_ONLY);
+    Tcl_DStringResult (interp, &resultBuf);
 
     Tcl_DStringFree (&callback);
+    Tcl_DStringFree (&resultBuf);
+    Tcl_DStringFree (&errorInfoBuf);
+    Tcl_DStringFree (&errorCodeBuf);
 }
 
-/*-----------------------------------------------------------------------------
+/*
+ *-----------------------------------------------------------------------------
  * CmdTraceRoutine --
  *
  *  Routine called by Tcl_Eval to trace a command.
@@ -362,12 +356,8 @@ CmdTraceRoutine (clientData, interp, level, command, cmdProc, cmdClientData,
     traceInfo_pt  infoPtr = (traceInfo_pt) clientData;
     int           procLevel;
 
-    /*
-     * If we are in an error.  
-     */
-    if (infoPtr->inTrace || (infoPtr->errorStatePtr != NULL)) {
+    if (infoPtr->inTrace)
         return;
-    }
     infoPtr->inTrace = TRUE;
 
     if (infoPtr->procCalls) {
@@ -390,8 +380,9 @@ CmdTraceRoutine (clientData, interp, level, command, cmdProc, cmdClientData,
     infoPtr->inTrace = FALSE;
 }
 
-/*-----------------------------------------------------------------------------
- * Tcl_CmdtraceObjCmd --
+/*
+ *-----------------------------------------------------------------------------
+ * Tcl_CmdtraceCmd --
  *
  * Implements the TCL trace command:
  *     cmdtrace level|on ?noeval? ?notruncate? ?procs? ?fileid? ?command cmd?
@@ -400,28 +391,26 @@ CmdTraceRoutine (clientData, interp, level, command, cmdProc, cmdClientData,
  *-----------------------------------------------------------------------------
  */
 static int
-TclX_CmdtraceObjCmd (clientData, interp, objc, objv)
-    ClientData  clientData;
-    Tcl_Interp *interp;
-    int         objc;
-    Tcl_Obj    *CONST objv[];
+Tcl_CmdtraceCmd (clientData, interp, argc, argv)
+    ClientData    clientData;
+    Tcl_Interp   *interp;
+    int           argc;
+    char        **argv;
 {
     traceInfo_pt  infoPtr = (traceInfo_pt) clientData;
-    int idx;
-    char *argStr, *callback;
-    Tcl_Obj *channelId;
+    int           idx;
+    char         *fileHandle, *callback;
 
-    if (objc < 2)
+    if (argc < 2)
         goto argumentError;
-    argStr = Tcl_GetStringFromObj (objv [1], NULL);
 
     /*
      * Handle `depth' sub-command.
      */
-    if (STREQU (argStr, "depth")) {
-        if (objc != 2)
+    if (STREQU (argv[1], "depth")) {
+        if (argc != 2)
             goto argumentError;
-        Tcl_SetIntObj (Tcl_GetObjResult (interp),  infoPtr->depth);
+        sprintf(interp->result, "%d", infoPtr->depth);
         return TCL_OK;
     }
 
@@ -433,8 +422,8 @@ TclX_CmdtraceObjCmd (clientData, interp, objc, objv)
     /*
      * Handle off sub-command.
      */
-    if (STREQU (argStr, "off")) {
-        if (objc != 2)
+    if (STREQU (argv[1], "off")) {
+        if (argc != 2)
             goto argumentError;
         return TCL_OK;
     }
@@ -443,109 +432,99 @@ TclX_CmdtraceObjCmd (clientData, interp, objc, objv)
     infoPtr->noTruncate = FALSE;
     infoPtr->procCalls  = FALSE;
     infoPtr->channel    = NULL;
-    channelId           = NULL;
+    fileHandle          = NULL;
     callback            = NULL;
 
-    if (STREQU (argStr, "on")) {
-        infoPtr->depth = MAXINT;
-    } else {
-        if (Tcl_GetIntFromObj (interp, objv [1], &(infoPtr->depth)) != TCL_OK)
-            return TCL_ERROR;
-    }
-
-    for (idx = 2; idx < objc; idx++) {
-        argStr = Tcl_GetStringFromObj (objv [idx], NULL);
-        if (STREQU (argStr, "notruncate")) {
+    for (idx = 2; idx < argc; idx++) {
+        if (STREQU (argv[idx], "notruncate")) {
             if (infoPtr->noTruncate)
                 goto argumentError;
             infoPtr->noTruncate = TRUE;
             continue;
         }
-        if (STREQU (argStr, "noeval")) {
+        if (STREQU (argv[idx], "noeval")) {
             if (infoPtr->noEval)
                 goto argumentError;
             infoPtr->noEval = TRUE;
             continue;
         }
-        if (STREQU (argStr, "procs")) {
+        if (STREQU (argv[idx], "procs")) {
             if (infoPtr->procCalls)
                 goto argumentError;
             infoPtr->procCalls = TRUE;
             continue;
         }
-        if (STRNEQU (argStr, "std", 3) || 
-                STRNEQU (argStr, "file", 4)) {
-            if (channelId != NULL)
+        if (STRNEQU (argv [idx], "std", 3) || 
+                STRNEQU (argv [idx], "file", 4)) {
+            if (fileHandle != NULL)
                 goto argumentError;
             if (callback != NULL)
                 goto mixCommandAndFile;
-            channelId = objv [idx];
+            fileHandle = argv [idx];
             continue;
         }
-        if (STREQU (argStr, "command")) {
+        if (STREQU (argv [idx], "command")) {
             if (callback != NULL)
                 goto argumentError;
-            if (channelId != NULL)
+            if (fileHandle != NULL)
                 goto mixCommandAndFile;
-            if (idx == objc - 1)
+            if (idx == argc - 1)
                 goto missingCommand;
-            callback = Tcl_GetStringFromObj (objv [++idx], NULL);
+            callback = argv [++idx];
             continue;
         }
         goto invalidOption;
     }
 
+    if (STREQU (argv[1], "on")) {
+        infoPtr->depth = MAXINT;
+    } else {
+        if (Tcl_GetInt (interp, argv[1], &(infoPtr->depth)) != TCL_OK)
+            return TCL_ERROR;
+    }
+
     if (callback != NULL) {
         infoPtr->callback = ckstrdup (callback);
-        infoPtr->errorAsyncHandler =
-            Tcl_AsyncCreate (TraceCallbackErrorHandler, 
-                             (ClientData) infoPtr);
-
     } else {
-        if (channelId == NULL) {
-            infoPtr->channel = TclX_GetOpenChannel (interp,
-                                                    "stdout",
-                                                    TCL_WRITABLE);
-        } else {
-            infoPtr->channel = TclX_GetOpenChannelObj (interp,
-                                                       channelId,
-                                                       TCL_WRITABLE);
-        }
+        if (fileHandle == NULL)
+            fileHandle = "stdout";
+        infoPtr->channel = TclX_GetOpenChannel (interp,
+                                                fileHandle,
+                                                TCL_WRITABLE);
         if (infoPtr->channel == NULL)
             return TCL_ERROR;
     }
-    infoPtr->traceId =
-        Tcl_CreateTrace (interp,
-                         infoPtr->depth,
-                         (Tcl_CmdTraceProc*) CmdTraceRoutine,
-                         (ClientData) infoPtr);
+    infoPtr->traceHolder = Tcl_CreateTrace (interp,
+                                            infoPtr->depth,
+                                            CmdTraceRoutine,
+                                            (ClientData) infoPtr);
     return TCL_OK;
 
   argumentError:
-    TclX_AppendObjResult (interp, tclXWrongArgs, objv [0], 
-                          " level | on ?noeval? ?notruncate? ?procs?",
-                          "?fileid? ?command cmd? | off | depth",
-                          (char *) NULL);
+    Tcl_AppendResult (interp, tclXWrongArgs, argv [0], 
+                      " level | on ?noeval? ?notruncate? ?procs?",
+                      "?fileid? ?command cmd? | off | depth", (char *) NULL);
     return TCL_ERROR;
 
   missingCommand:
-    TclX_AppendObjResult (interp, "command option requires an argument",
-                          (char *) NULL);
+    Tcl_AppendResult (interp, "command option requires an argument",
+                      (char *) NULL);
     return TCL_ERROR;
 
   mixCommandAndFile:
-    TclX_AppendObjResult (interp, "can not specify both the command option ",
-                          "and a file handle", (char *) NULL);
+    Tcl_AppendResult (interp, "can not specify both the command option and ",
+                      "a file handle", (char *) NULL);
     return TCL_ERROR;
 
   invalidOption:
-    TclX_AppendObjResult (interp, "invalid option: expected ",
-                          "one of \"noeval\", \"notruncate\", \"procs\", ",
-                          "\"command\", or a file id", (char *) NULL);
+    Tcl_AppendResult (interp, "invalid option: expected ",
+                      "one of \"noeval\", \"notruncate\", \"procs\", ",
+                      "\"command\", or a file id", (char *) NULL);
     return TCL_ERROR;
 }
 
-/*-----------------------------------------------------------------------------
+/*
+ *-----------------------------------------------------------------------------
  * DebugCleanUp --
  *
  *  Release the debug data area when the interpreter is deleted.
@@ -562,40 +541,35 @@ DebugCleanUp (clientData, interp)
     ckfree ((char *) infoPtr);
 }
 
-/*-----------------------------------------------------------------------------
- * TclX_DebugInit --
+/*
+ *-----------------------------------------------------------------------------
+ * Tcl_InitDebug --
  *
  *  Initialize the TCL debugging commands.
  *-----------------------------------------------------------------------------
  */
 void
-TclX_DebugInit (interp)
+Tcl_InitDebug (interp)
     Tcl_Interp *interp;
 {
     traceInfo_pt infoPtr;
 
     infoPtr = (traceInfo_pt) ckalloc (sizeof (traceInfo_t));
 
-    infoPtr->interp = interp;
-    infoPtr->traceId = NULL;
-    infoPtr->inTrace = FALSE;
-    infoPtr->noEval = FALSE;
-    infoPtr->noTruncate = FALSE;
-    infoPtr->procCalls = FALSE;
-    infoPtr->depth = 0;
-    infoPtr->callback = NULL;
-    infoPtr->errorStatePtr = NULL;
-    infoPtr->errorAsyncHandler = NULL;
-    infoPtr->channel = NULL;
+    infoPtr->interp      = interp;
+    infoPtr->traceHolder = NULL;
+    infoPtr->inTrace     = FALSE;
+    infoPtr->noEval      = FALSE;
+    infoPtr->noTruncate  = FALSE;
+    infoPtr->procCalls   = FALSE;
+    infoPtr->depth       = 0;
+    infoPtr->callback    = NULL;
+    infoPtr->channel     = NULL;
 
     Tcl_CallWhenDeleted (interp, DebugCleanUp, (ClientData) infoPtr);
 
-    Tcl_CreateObjCommand (interp, "cmdtrace",
-                          TclX_CmdtraceObjCmd, 
-                          (ClientData) infoPtr,
-                          (Tcl_CmdDeleteProc*) NULL);
+    Tcl_CreateCommand (interp, "cmdtrace", Tcl_CmdtraceCmd, 
+                       (ClientData) infoPtr, (void (*)()) NULL);
 }
 
 
-
-
