@@ -5,7 +5,7 @@
  * Tcl handles.  Provides a mechanism for managing expandable tables that are
  * addressed by textual handles.
  *-----------------------------------------------------------------------------
- * Copyright 1991-1999 Karl Lehenbauer and Mark Diekhans.
+ * Copyright 1991-1996 Karl Lehenbauer and Mark Diekhans.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation for any purpose and without fee is hereby granted, provided
@@ -14,7 +14,7 @@
  * software for any purpose.  It is provided "as is" without express or
  * implied warranty.
  *-----------------------------------------------------------------------------
- * $Id: tclXhandles.c,v 8.6 1999/03/31 06:37:44 markd Exp $
+ * $Id: tclXhandles.c,v 7.0 1996/06/16 05:30:27 markd Exp $
  *-----------------------------------------------------------------------------
  */
 
@@ -99,16 +99,10 @@ static entryHeader_pt
 AllocEntry _ANSI_ARGS_((tblHeader_pt  tblHdrPtr,
                         int          *entryIdxPtr));
 
-static int
-HandleDecodeObj _ANSI_ARGS_((Tcl_Interp   *interp,
-                             tblHeader_pt  tblHdrPtr,
-                             CONST char   *handle));
-
 static int
 HandleDecode _ANSI_ARGS_((Tcl_Interp   *interp,
                           tblHeader_pt  tblHdrPtr,
                           CONST char   *handle));
-
 
 /*=============================================================================
  * LinkInNewEntries --
@@ -171,7 +165,7 @@ ExpandTable (tblHdrPtr, neededIdx)
             (tblHdrPtr->tableSize * tblHdrPtr->entrySize));
     LinkInNewEntries (tblHdrPtr, tblHdrPtr->tableSize, numNewEntries);
     tblHdrPtr->tableSize += numNewEntries;
-    ckfree ((char *) oldbodyPtr);
+    ckfree (oldbodyPtr);
     
 }
 
@@ -211,9 +205,6 @@ AllocEntry (tblHdrPtr, entryIdxPtr)
  * HandleDecode --
  *   Decode handle into an entry number.
  *
- *   Same as HandleDecode except it uses the object-based result
- *   mechanism if an error occurs.
- *
  * Parameters:
  *   o interp (I) - A error message may be returned in result.
  *   o tblHdrPtr (I) - A pointer to the table header.
@@ -233,52 +224,18 @@ HandleDecode (interp, tblHdrPtr, handle)
 
     if ((strncmp (tblHdrPtr->handleBase, (char *) handle, 
              tblHdrPtr->baseLength) != 0) ||
-             !TclX_StrToUnsigned (&handle [tblHdrPtr->baseLength], 10, 
+             !Tcl_StrToUnsigned (&handle [tblHdrPtr->baseLength], 10, 
                                  &entryIdx)) {
-        TclX_AppendObjResult (interp, "invalid ", tblHdrPtr->handleBase,
-                              " handle \"", handle, "\"", (char *) NULL);
+        Tcl_AppendResult (interp, "invalid ", tblHdrPtr->handleBase,
+                          " handle \"", handle, "\"", (char *) NULL);
         return -1;
     }
     return entryIdx;
-}
-
-/*=============================================================================
- * HandleDecodeObj --
- *   Decode handle into an entry number.
- *
- *   Same as HandleDecode except it uses the object-based result
- *   mechanism if an error occurs.
- *
- * Parameters:
- *   o interp (I) - A error message may be returned in result.
- *   o tblHdrPtr (I) - A pointer to the table header.
- *   o handle (I) - Handle to decode.
- * Returns:
- *   The entry index decoded from the handle, or a negative number if an error
- *   occured.
- *-----------------------------------------------------------------------------
- */
-static int
-HandleDecodeObj (interp, tblHdrPtr, handle)
-    Tcl_Interp   *interp;
-    tblHeader_pt  tblHdrPtr;
-    CONST char   *handle;
-{
-    unsigned entryIdx;
 
-    if ((strncmp (tblHdrPtr->handleBase, (char *) handle, 
-                  tblHdrPtr->baseLength) != 0) ||
-        !TclX_StrToUnsigned (&handle [tblHdrPtr->baseLength], 10, 
-                             &entryIdx)) {
-        TclX_AppendObjResult (interp, "invalid ", tblHdrPtr->handleBase,
-                              " handle \"", handle, "\"", (char *) NULL);
-        return -1;
-    }
-    return entryIdx;
 }
 
 /*=============================================================================
- * TclX_HandleTblInit --
+ * Tcl_HandleTblInit --
  *   Create and initialize a Tcl dynamic handle table.  The use count on the
  *   table is set to one.
  * Parameters:
@@ -291,7 +248,7 @@ HandleDecodeObj (interp, tblHdrPtr, handle)
  *-----------------------------------------------------------------------------
  */
 void_pt
-TclX_HandleTblInit (handleBase, entrySize, initEntries)
+Tcl_HandleTblInit (handleBase, entrySize, initEntries)
     CONST char *handleBase;
     int         entrySize;
     int         initEntries;
@@ -338,7 +295,7 @@ TclX_HandleTblInit (handleBase, entrySize, initEntries)
 }
 
 /*=============================================================================
- * TclX_HandleTblUseCount --
+ * Tcl_HandleTblUseCount --
  *   Alter the handle table use count by the specified amount, which can be
  *   positive or negative.  Amount may be zero to retrieve the use count.
  * Parameters:
@@ -349,7 +306,7 @@ TclX_HandleTblInit (handleBase, entrySize, initEntries)
  *-----------------------------------------------------------------------------
  */
 int
-TclX_HandleTblUseCount (headerPtr, amount)
+Tcl_HandleTblUseCount (headerPtr, amount)
     void_pt  headerPtr;
     int      amount;
 {
@@ -360,7 +317,7 @@ TclX_HandleTblUseCount (headerPtr, amount)
 }
 
 /*=============================================================================
- * TclX_HandleTblRelease --
+ * Tcl_HandleTblRelease --
  *   Decrement the use count on a Tcl dynamic handle table.  If the count
  * goes to zero or negative, then release the table.
  *
@@ -369,20 +326,20 @@ TclX_HandleTblUseCount (headerPtr, amount)
  *-----------------------------------------------------------------------------
  */
 void
-TclX_HandleTblRelease (headerPtr)
+Tcl_HandleTblRelease (headerPtr)
     void_pt headerPtr;
 {
     tblHeader_pt  tblHdrPtr = (tblHeader_pt) headerPtr;
 
     tblHdrPtr->useCount--;
     if (tblHdrPtr->useCount <= 0) {
-        ckfree ((char *) tblHdrPtr->bodyPtr);
+        ckfree (tblHdrPtr->bodyPtr);
         ckfree ((char *) tblHdrPtr);
     }
 }
 
 /*=============================================================================
- * TclX_HandleAlloc --
+ * Tcl_HandleAlloc --
  *   Allocate an entry and associate a handle with it.
  *
  * Parameters:
@@ -394,7 +351,7 @@ TclX_HandleTblRelease (headerPtr)
  *-----------------------------------------------------------------------------
  */
 void_pt
-TclX_HandleAlloc (headerPtr, handlePtr)
+Tcl_HandleAlloc (headerPtr, handlePtr)
     void_pt   headerPtr;
     char     *handlePtr;
 {
@@ -410,7 +367,7 @@ TclX_HandleAlloc (headerPtr, handlePtr)
 }
 
 /*=============================================================================
- * TclX_HandleXlate --
+ * Tcl_HandleXlate --
  *   Translate a handle to a entry pointer.
  *
  * Parameters:
@@ -422,7 +379,7 @@ TclX_HandleAlloc (headerPtr, handlePtr)
  *-----------------------------------------------------------------------------
  */
 void_pt
-TclX_HandleXlate (interp, headerPtr, handle)
+Tcl_HandleXlate (interp, headerPtr, handle)
     Tcl_Interp *interp;
     void_pt     headerPtr;
     CONST char *handle;
@@ -437,8 +394,8 @@ TclX_HandleXlate (interp, headerPtr, handle)
 
     if ((entryIdx >= tblHdrPtr->tableSize) ||
             (entryHdrPtr->freeLink != ALLOCATED_IDX)) {
-        TclX_AppendObjResult (interp, tblHdrPtr->handleBase, " is not open",
-                              (char *) NULL);
+        Tcl_AppendResult (interp, tblHdrPtr->handleBase, " is not open",
+                          (char *) NULL);
         return NULL;
     }     
 
@@ -447,46 +404,7 @@ TclX_HandleXlate (interp, headerPtr, handle)
 }
 
 /*=============================================================================
- * TclX_HandleXlateObj --
- *   Translate an object containing a handle name to a entry pointer.
- *
- * Parameters:
- *   o interp (I) - A error message may be returned in result.
- *   o headerPtr (I) - A pointer to the table header.
- *   o handleObj (I) - The object containing the handle assigned to the entry.
- * Returns:
- *   A pointer to the entry, or NULL if an error occured.
- *-----------------------------------------------------------------------------
- */
-void_pt
-TclX_HandleXlateObj (interp, headerPtr, handleObj)
-    Tcl_Interp *interp;
-    void_pt     headerPtr;
-    Tcl_Obj *handleObj;
-{
-    tblHeader_pt   tblHdrPtr = (tblHeader_pt)headerPtr;
-    entryHeader_pt entryHdrPtr;
-    int            entryIdx;
-    char          *handle;
-
-    handle = Tcl_GetStringFromObj (handleObj, NULL);
-    
-    if ((entryIdx = HandleDecodeObj (interp, tblHdrPtr, handle)) < 0)
-        return NULL;
-    entryHdrPtr = TBL_INDEX (tblHdrPtr, entryIdx);
-
-    if ((entryIdx >= tblHdrPtr->tableSize) ||
-            (entryHdrPtr->freeLink != ALLOCATED_IDX)) {
-        TclX_AppendObjResult (interp, tblHdrPtr->handleBase, 
-                              " is not open", (char *) NULL);
-        return NULL;
-    }     
-
-    return USER_AREA (entryHdrPtr);
-}
-
-/*=============================================================================
- * TclX_HandleWalk --
+ * Tcl_HandleWalk --
  *   Walk through and find every allocated entry in a table.  Entries may
  *   be deallocated during a walk, but should not be allocated.
  *
@@ -500,7 +418,7 @@ TclX_HandleXlateObj (interp, headerPtr, handleObj)
  *-----------------------------------------------------------------------------
  */
 void_pt
-TclX_HandleWalk (headerPtr, walkKeyPtr)
+Tcl_HandleWalk (headerPtr, walkKeyPtr)
     void_pt   headerPtr;
     int      *walkKeyPtr;
 {
@@ -526,7 +444,7 @@ TclX_HandleWalk (headerPtr, walkKeyPtr)
 }
 
 /*=============================================================================
- * TclX_WalkKeyToHandle --
+ * Tcl_WalkKeyToHandle --
  *   Convert a walk key, as returned from a call to Tcl_HandleWalk into a
  *   handle.  The Tcl_HandleWalk must have succeeded.
  * Parameters:
@@ -537,7 +455,7 @@ TclX_HandleWalk (headerPtr, walkKeyPtr)
  *-----------------------------------------------------------------------------
  */
 void
-TclX_WalkKeyToHandle (headerPtr, walkKey, handlePtr)
+Tcl_WalkKeyToHandle (headerPtr, walkKey, handlePtr)
     void_pt   headerPtr;
     int       walkKey;
     char     *handlePtr;
@@ -549,7 +467,7 @@ TclX_WalkKeyToHandle (headerPtr, walkKey, handlePtr)
 }
 
 /*=============================================================================
- * TclX_HandleFree --
+ * Tcl_HandleFree --
  *   Frees a handle table entry.
  *
  * Parameters:
@@ -558,7 +476,7 @@ TclX_WalkKeyToHandle (headerPtr, walkKey, handlePtr)
  *-----------------------------------------------------------------------------
  */
 void
-TclX_HandleFree (headerPtr, entryPtr)
+Tcl_HandleFree (headerPtr, entryPtr)
     void_pt headerPtr;
     void_pt entryPtr;
 {
@@ -575,5 +493,3 @@ TclX_HandleFree (headerPtr, entryPtr)
     
 }
 
-
-
