@@ -5,7 +5,7 @@
 # mechanism.  This helps if something is broken with auto-load so the
 # build at least completes.
 #------------------------------------------------------------------------------
-# Copyright 1992-1999 Karl Lehenbauer and Mark Diekhans.
+# Copyright 1992-1996 Karl Lehenbauer and Mark Diekhans.
 #
 # Permission to use, copy, modify, and distribute this software and its
 # documentation for any purpose and without fee is hereby granted, provided
@@ -14,14 +14,11 @@
 # software for any purpose.  It is provided "as is" without express or
 # implied warranty.
 #------------------------------------------------------------------------------
-# $Id: genindex.tcl,v 8.4 1999/03/31 06:37:59 markd Exp $
+# $Id: genindex.tcl,v 7.0 1996/06/16 05:34:37 markd Exp $
 #------------------------------------------------------------------------------
 #
 
-#cmdtrace on
 global tclx_library
-source [file join $tclx_library buildidx.tcl]
+source $tclx_library/buildidx.tcl
 
 buildpackageindex $argv
-
-
