@@ -3,10 +3,10 @@
 #
 # Tcl program to copy files during the installation of Tcl.  This is used
 # because "copy -r" is not ubiquitous.  It also adds some minor additional
-# functionality.
+# functionallity.
 #
 #------------------------------------------------------------------------------
-# Copyright 1992-1999 Karl Lehenbauer and Mark Diekhans.
+# Copyright 1992-1996 Karl Lehenbauer and Mark Diekhans.
 #
 # Permission to use, copy, modify, and distribute this software and its
 # documentation for any purpose and without fee is hereby granted, provided
@@ -15,7 +15,7 @@
 # software for any purpose.  It is provided "as is" without express or
 # implied warranty.
 #------------------------------------------------------------------------------
-# $Id: instcopy.tcl,v 8.7 2002/11/12 21:35:31 karll Exp $
+# $Id: instcopy.tcl,v 7.0 1996/06/16 05:34:39 markd Exp $
 #------------------------------------------------------------------------------
 #
 # It is run in the following manner:
@@ -25,16 +25,12 @@
 #
 #  o -filename - If specified, then the last file is the name of a file rather
 #    than a directory. 
-#  o -bin - Force file to be copied without translation. (not implemented).
 #  o files - List of files to copy. If one of directories are specified, they
 #    are copied.
 #  o targetdir - Target directory to copy the files to.  If the directory does
 #    not exist, it is created (including parent directories).
 #::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 
-package require Tclx
-source [file join [file dirname [info script]] buildutil.tcl]
-
 #------------------------------------------------------------------------------
 # Usage --
 #
@@ -60,12 +56,14 @@ proc DoACopy {file target mode} {
     if {$mode == "FILENAME"} {
         set targetDir [file dirname $target]
         if [file exists $target] {
-            file delete -force $target
+            unlink $target
         }
     } else {
         set targetDir $target
     }
-    file mkdir $targetDir
+    if ![file exists $targetDir] {
+        mkdir -path  $targetDir
+    }
 
     if [file isdirectory $file] {
         CopyDir $file $target
@@ -86,29 +84,19 @@ if {$argc < 2} {
     Usage "Not enough arguments"
 }
 
-set mode {}
-set binary 0
-while {[string match -* [lindex $argv 0]]} {
-    set flag [lvarpop argv]
-    incr argc -1
-    switch -exact -- $flag {
-        -filename {
-            set mode FILENAME
-        }
-        -bin {
-            set binary 1
-        }
-        default {
-            puts stderr "unknown flag"
-        }
+switch -exact -- [lindex $argv 0] {
+    -filename {
+        set mode FILENAME
+        lvarpop argv
+        incr argc -1
+    }
+    default {
+        set mode {}
     }
 }
 
-set files {}
-foreach file [lrange $argv 0 [expr $argc-2]] {
-    lappend files [eval file join [file split $file]]
-}
-set targetDir [eval file join [file split [lindex $argv [expr $argc-1]]]]
+set files [lrange $argv 0 [expr $argc-2]]
+set targetDir [lindex $argv [expr $argc-1]]
 
 if {[file exists $targetDir] && ![file isdirectory $targetDir] &&
     ($mode != "FILENAME")} {
@@ -125,5 +113,3 @@ if [catch {
     puts stderr "Error: $msg"
     exit 1
 }
-
-
